// src/electron/main.ts
// 需要先載入
import "module-alias/register";

// // 必載入
// import "../Utils/catan.ts";

import dotenv from 'dotenv';
import { app, BrowserWindow, ipcMain } from 'electron';
import fs from 'fs';
import * as path from 'path';
import { WebSocketServer } from 'ws';
import { BaseEnumerator } from "../CatanEngine/CoroutineV2/Core/BaseEnumerator";
import { NetConnector } from "../script/Engine/CatanEngine/NetManagerV2/NetConnector";
import { slotJsons } from "../Utils/tools";

onload();

function onload() {
	BaseEnumerator.Init();
	setLog();
	// Load environment variables from .env file
	const filePath = ".env.dev";
	// 检查文件是否存在
	if (fs.existsSync(filePath)) {
		dotenv.config({ path: `.env.dev` });
	} else {
		dotenv.config({ path: path.resolve(__dirname, `../../.env.prod`) });
	}
}

let server: WebSocketServer | null = null;
let port: number = process.env.PORT ? parseInt(process.env.PORT, 10) : 8080;  // 默认端口为 8080

function createWindow() {
	const win = new BrowserWindow({
		width: 800,
		height: 600,
		webPreferences: {
			preload: path.join(__dirname, 'preload.js'),
			contextIsolation: true,
			nodeIntegration: false,
		},
	});

	win.loadFile(path.join(__dirname, 'index.html'));
	// win.webContents.openDevTools();

	return win;
}

let mainWindow: BrowserWindow | null = null;

app.whenReady().then(() => {
	mainWindow = createWindow();

	app.on('activate', () => {
		if (BrowserWindow.getAllWindows().length === 0) {
			createWindow();
		}
	});
});

app.on('window-all-closed', () => {
	if (process.platform !== 'darwin') {
		app.quit();
	}
});

// 启动 WebSocket 服务器
ipcMain.on('start-websocket', (event: Electron.IpcMainEvent, portNumber: number) => {
	if (server) {
		closeServer(event);
	}

	port = portNumber || 8080;

	server = new WebSocketServer({ port });

	server.on('connection', NetConnector.OnWebSocketConnection);

	event.reply('websocket-status', `WebSocket server started on port ${port}`);
});

// 关闭 WebSocket 服务器
ipcMain.on('stop-websocket', (event: Electron.IpcMainEvent) => {
	closeServer(event);
});

// 打开开发者工具
ipcMain.on('json-reload', () => {
	NetConnector.clients.forEach(client => {
		slotJsons.clear();
	});
	console.log(`重載成功`);
});

// 打开开发者工具
ipcMain.on('open-devtools', () => {
	mainWindow.webContents.openDevTools();
});

function closeServer(event: Electron.IpcMainEvent) {
	if (server) {
		server.clients.forEach(client => {
			if (client.readyState === 1) {
				client.close();
			}
		});
		server.close((err) => {
			if (err) {
				event.reply('websocket-status', 'Failed to stop WebSocket server');
			}
		});
		event.reply('websocket-status', 'WebSocket server stopped');
	} else {
		event.reply('websocket-status', 'No WebSocket server is running');
	}
}

// Create a function to send log messages to the renderer
function sendLogToRenderer(window: BrowserWindow, message: string, color: string) {
	window.webContents.send('log-message', [message, color]);
}

function setLog() {
	// 重寫 console.log 方法以便將訊息發送到渲染進程
	const originalConsoleLog = console.log;
	console.log = (...args: any[]) => {
		const message = args.join(' ');
		originalConsoleLog(message);  // 保留原始行為
		if (BrowserWindow.getAllWindows().length > 0) {
			sendLogToRenderer(BrowserWindow.getAllWindows()[0], message, "green");  // 發送訊息到渲染進程
		}
	};

	// 重写 console.warn
	const originalConsoleWarn = console.warn;
	console.warn = (...args: any[]) => {
		const message = args.join(' ');
		originalConsoleWarn(message);
		if (BrowserWindow.getAllWindows().length > 0) {
			sendLogToRenderer(BrowserWindow.getAllWindows()[0], message, "yellow");  // 發送訊息到渲染進程
		}
	};

	// 重写 console.error
	const originalConsoleError = console.error;
	console.error = (...args: any[]) => {
		const message = args.join(' ');
		originalConsoleError(message);
		if (BrowserWindow.getAllWindows().length > 0) {
			sendLogToRenderer(BrowserWindow.getAllWindows()[0], message, "red");  // 發送訊息到渲染進程
		}
	};
}